clear matrix
clear mata
clear      
set more off, perm
capture log close
*=========================================================;
* Set working directory to the folder containing this file
cd /your/working/directory
*=========================================================;
# delimit ;
*=================================================================================================;
*=================================================================================================;
*=================================================================================================;
* File-Name:       	session1.do
* Date:            	17/07/15
* Author:          	DD	
* Machine:         	DD's MacBook Pro
*=================================================================================================;
*=================================================================================================;
* Programs and locals
*=================================================================================================;
* define a bunch of helpful locals;
local graphr "graphr(fc(white) lc(white) ifc(white) ilc(white))";
local graphrBlack "graphr(fc(white) lc(black) ifc(white) ilc(white))";
local grcom "ycommon xcommon imargin(small)";
*=================================================================================================;
*=================================================================================================;
*=================================================================================================;
* Session 1 -- Part 1
*=================================================================================================;
* Distributions
*=================================================================================================;
* t-distribution;
gr tw 
	(function y=normalden(x), lc(red) lw(vthick) range(-4 4))
	(function y=tden(1,x), lc(gs12) lw(thick) range(-4 4))
	(function y=tden(4,x), lc(gray) lw(thick) range(-4 4))
	(function y=tden(29,x), lc(black) lw(thick) range(-4 4)), 
`graphr' xti("{it: theta}") ysc(range(0 .5)) ylab(0(.1).5)
yti("Density") ti("z- and t-distribution", col(black))
leg(order(1 "normal" 2 "t, n=2" 3 "t, n=5" 4 "t, n=30") rows(1));
gr export probFunction_normalVsTDistr.pdf, replace;

* type of distributions;
gr tw 	
(function y=normalden(x), lc(red) lw(vthick) range(-4 4))
(function y=chi2den(3,x), lc(gs12) lw(thick) range(-4 4))
(function y=chi2den(10,x), lc(gs12) lw(thick) range(-4 4)), 
`graphr' xti("{it: theta}") ysc(range(0 .5)) ylab(0(.1).5)
yti("Density") ti("normal and skewed distributions", col(black))
leg(order(1 "normal" 2 "chi^2, df=3" 3 "chi^2, df=10") rows(1));
gr export typeOfDistributions.pdf, replace; 

local TwithDf5 = invttail(5,.05);
local TwithDf30 = invttail(30,.05);
local TwithDf100 = invttail(100,.05);

gr tw 
(function y=tden(5,x), lc(red) range(-4 4))
(function y=tden(30,x), lc(black) range(-4 4))
(function y=tden(100,x), lc(blue) range(-4 4)), 
`graphr' xti("{it: theta}") ysc(range(0 .5)) ylab(0(.1).5)
xline(`TwithDf5', lc(red))
xline(`TwithDf30', lc(black))
xline(`TwithDf100', lc(blue))
legend(order(1 "n=5" 2 "n=30" 3 "n=100") rows(1)) yti(Density)
ti("Distribution of T-statistic and t(n,0.05)");
gr export pdf_tDistrPlusAlphaLine.pdf, replace;  

*=================================================================================================;
* Exploratory data analysis
*=================================================================================================;
* Influential observations; 
use `data'/fakeData, clear; 

* effect of deleting single observation, Cook's D;
reg var;
predict cooksD, cooksd;
gr tw 
	(scatter cooksD var, mc(black) `graphr' yti(Cook's D)); 
gr export scatter_cooksDVsVar.pdf, replace; 

* effect of single observation on overall mean;
g meanMinusVarRaw = .;
g influenceMeasure = .;

vallist var;
foreach val in `r(list)' {;
    replace meanMinusVarRaw = var if(var~=`val');
    egen meanMinusVar = mean(meanMinusVarRaw);
    count if(var~=`val');
    local n = r(N); 
    replace influenceMeasure = 1/(`n'+1)*(var + meanMinusVar) if(var==`val');
    replace meanMinusVar = .;
    drop meanMinusVar;
};

gr tw
    (connected influenceMeasure var, mc(black) lc(black) sort), 
    `graphr' yti(Influence) ysc(range(0 2)) ylab(0(.5)2);
gr export connected_influenceMeasureVsVar.pdf, replace; 
	
extremes var;

*=================================================================================================;
* Randomization inference
*=================================================================================================;
use fakeData, clear;
keep in 1/6;
ttest var, by(cat);

* Derive randomization distribution;
clear;
local N = 2^6;
set obs `N';
egen T_1 = seq(), from(0) to(1);
bysort T_1: egen T_2 = seq(), from(0) to(1);
bysort T_1 T_2: egen T_3 = seq(), from(0) to(1);
bysort T_1 T_2 T_3: egen T_4 = seq(), from(0) to(1);
bysort T_1 T_2 T_3 T_4: egen T_5 = seq(), from(0) to(1);
bysort T_1 T_2 T_3 T_4 T_5: egen T_6 = seq(), from(0) to(1);

egen rowSum = rowtotal(T_1-T_6);
keep if(rowSum==3);
drop rowSum;

g Y_1 = 5;
g Y_2 = 3;
g Y_3 = 3;
g Y_4 = 15;
g Y_5 = 12;
g Y_6 = 6;

g N_T = 3;
g N_C = 3;

g meanTreatment = (T_1*Y_1 + T_2*Y_2 + T_3*Y_3 + T_4*Y_4 + T_5*Y_5 + T_6*Y_6)/N_T; 
g meanControl = ((1-T_1)*Y_1 + (1-T_2)*Y_2 + (1-T_3)*Y_3 + (1-T_4)*Y_4 + (1-T_5)*Y_5 + (1-T_6)*Y_6)/N_C;
g ate = meanTreatment - meanControl;

_pctile ate, p(2.5 97.5);
local lb95 = r(r1);
local ub95 = r(r2);

hist ate, `graphr' discret xline(-.67, lc(red) lw(thick)) xline(`lb95', lc(black) lp(dash))
xline(`ub95', lc(black) lp(dash));
gr export hist_ateRIDistribution.pdf, replace; 

*=================================================================================================;
* Session 1 -- Part 2
*=================================================================================================;
* Simulations
*=================================================================================================;
* simulate;
program drop _all;
program define normalDistribution, rclass;
	syntax [, obs(integer 1) mean(real 0) sd(real 1)];
	drop _all;
	set obs `obs';
	tempvar mu;
	g `mu' = rnormal(`mean',`sd');
	sum `mu';
	return scalar mu = r(mean);
end;

foreach s in 5 50 500 5000 {;
	simulate mu=r(mu), reps(`s') seed(010101) saving(sim, replace): 
	normalDistribution, obs(15) mean(0) sd(1); 
	use sim, clear;
	hist mu, `graphr' col(blue) name(hist`s', replace)
	ti("Histogram of mu for S = `s'", col(black));
};

gr combine hist5 hist50 hist500 hist5000, `graphr' `grcom' rows(2); 
gr export hist_simNormalDistrStata.pdf, replace;

* postfile;
set seed 010101;
local obs = 15;
local mean = 0;
local sd = 1;
local nSimsList = "5 50 500 5000";
foreach s in `nSimsList' {;
	tempname normalDistribution;
	postfile `normalDistribution' mu using sim, replace;
	forvalue i = 1/`s' {;
		drop _all;
		set obs `obs';
		tempvar mu;
		g `mu' = rnormal(`mean',`sd');
		sum `mu'; 
		post `normalDistribution' (r(mean));
	};
	postclose `normalDistribution';

	use sim, clear; 
	hist mu, `graphr' col(blue) name(hist`s', replace)
	ti("Histogram of mu for S = `s'", col(black));
};

gr combine hist5 hist50 hist500 hist5000, `graphr' `grcom' rows(2); 
gr export hist_postfileNormalDistrStata.pdf, replace;

*=================================================================================================;
* Randomization inference
*=================================================================================================;
* permute
*=========================================================;
use fakeData, clear;
keep in 1/6;
ttest var, by(cat);

permute cat ate=(r(mu_2)-r(mu_1)), reps(20) seed(010101) sav(permuteTTest, replace) nodots 
nowarn nodrop left: ttest var, by(cat); 
preserve;
use permuteTTest, clear; 
_pctile ate, p(2.5 97.5);
local lb95 = r(r1);
local ub95 = r(r2);
hist ate, `graphr' discret xline(-.67, lc(red) lw(thick)) xline(`lb95', lc(black) lp(dash))
	xline(`ub95', lc(black) lp(dash)) xti(ate);
gr export hist_permutedATE_20Draws.pdf, replace;	  
restore;

permute cat ate=(r(mu_2)-r(mu_1)), reps(1000) sav(permuteTTest, replace) nodots nowarn nodrop
left: ttest var, by(cat); 

*preserve;
use permuteTTest, clear; 
asdf
_pctile ate, p(2.5 97.5);
local lb95 = r(r1);
local ub95 = r(r2);
hist ate, `graphr' discret xline(-.67, lc(red) lw(thick)) xline(`lb95', lc(black) lp(dash))
	xline(`ub95', lc(black) lp(dash)) xti(ate);
gr export hist_permutedATE_1000Draws.pdf, replace;	  
restore;

*=========================================================;
* tsrtest
*=========================================================;
use fakeData, clear;
keep in 1/6;
ttest var, by(cat);

program drop _all;
program diffInMeans, rclass;
	sum var if(cat==0);
    local control=r(mean);
	sum var if(cat==1);
    local treatment=r(mean);
    return scalar ate = `treatment'-`control';
end;
tsrtest cat r(ate), reps(1000) seed(01010) nullvalue(-.67) exact: diffInMeans;

*=================================================================================================;
* Performance of standard estimators in small samples
*=================================================================================================;
* Robustness
*=========================================================;
* Task 1;
* Extract t-statistic when assumptions for t-test about underlying population are met;
program drop _all;
program testRobustness, rclass;
	syntax, [ 
		obs(integer 1) n1(integer 0) n2(integer 0) mean1(real 0) mean2(real 0) sd1(real 1) sd2(real 1)
		];
	drop _all;
	local N = `n1' + `n2';
	set obs `N';
	tempname var rdmCat cat;
	g `cat' = 0 in 1/`n1';
	local n2Start = `n1' + 1;
	replace `cat' = 1 in `n2Start'/`N';
	g `var' = rnormal(`mean1',`sd1') if(`cat'==0);
	replace `var' = rnormal(`mean2',`sd2') if(`cat'==1);
	ttest `var', by(`cat');
	return scalar t = r(t);
end;

* now simulate 1000 runs of a t-test with N=4, 10, 20, 40 and juxtapose with theoretical t-distribution;
foreach n in 2 5 10 20 {;
	simulate t=r(t), reps(1000) nodots seed(010101): 
		testRobustness, n1(`n') n2(`n') mean1(0) sd1(1) mean2(0) sd2(1);
	local df = 2*`n'-2;
	local ti = 2*`n';
	gr tw
		(hist t, col(red))
		(function y=tden(`df',x), lc(gs12) lw(thick) range(-4 4)), 
	`graphr' xti("") leg(off) ti(N=`ti', col(black)) ysc(range(0 .5)) ylab(0(.1).5)  
	name(plot`n', replace);	
	};

gr combine plot2 plot5 plot10 plot20, `graphr' ycommon imargin(small) b1(T);
gr export hist_simulatedT_byN.pdf, replace;	

* Task 2;
* Extracts t-statistic when assumptions about underlying population are violated;
program drop _all;
program testRobustness, rclass;
	syntax, [ 
		obs(integer 1) n1(integer 0) n2(integer 0) mean1(real 0) mean2(real 0) sd1(real 1) sd2(real 1)
		];
	drop _all;
	local N = `n1' + `n2';
	set obs `N';
	tempname var cat rdm outlier;
	g `cat' = 0 in 1/`n1';
	local n2Start = `n1' + 1;
	replace `cat' = 1 in `n2Start'/`N';
	g `var' = rnormal(`mean1',`sd1') if(`cat'==0);
	replace `var' = rnormal(`mean2',`sd2') if(`cat'==1);
	g `rdm' = runiform();
	g `outlier' = 20 if(`rdm'<=.2);
	replace `outlier' = 0 if(`rdm'>.2);
	replace `var' = `var' + `outlier';
	ttest `var', by(`cat');
	return scalar t = r(t);
end;

foreach n in 2 5 10 20 {;
	simulate t=r(t), reps(1000) nodots seed(010101): 
		testRobustness, n1(`n') n2(`n') mean1(0) sd1(10) mean2(0) sd2(1);
	local df = 2*`n'-2;
	local ti = 2*`n';
	gr tw
		(hist t, col(red))
		(function y=tden(`df',x), lc(gs12) lw(thick) range(-4 4)), 
	`graphr' xti("") leg(off) ti(N=`ti', col(black)) ysc(range(0 .5)) ylab(0(.1).5) 
	name(plot`n', replace);	
	};

gr combine plot2 plot5 plot10 plot20, `graphr' ycommon imargin(small) b1(T);
gr export hist_simulatedT_byNAndSkewedPlusOutliers.pdf, replace;	

*=========================================================;
* Small type I error
*=========================================================;
* Extract the H_0 rejection rate for various tests and sample sizes;
program drop _all;
program testSmallTypeIError, rclass;
syntax, [ 
	obs(integer 1) n1(integer 0) n2(integer 0) mean1(real 0) mean2(real 0) sd1(real 1) sd2(real 1)
	alpha(real 0)
	];
drop _all;
local N = `n1' + `n2';
set obs `N';
tempname var cat rdm outlier tTest ranksum permute;
g `cat' = 0 in 1/`n1';
local n2Start = `n1' + 1;
replace `cat' = 1 in `n2Start'/`N';
	g `var' = rnormal(`mean1',`sd1')  if(`cat'==0);
	replace `var' = rnormal(`mean2',`sd2')  if(`cat'==1);
	g `rdm' = runiform();
	g `outlier' = 20 if(`rdm'<=.2);
	replace `outlier' = 0 if(`rdm'>.2);
	replace `var' = `var' + `outlier';
	ttest `var', by(`cat');
	g `tTest' = 0;
	replace `tTest' = 1 if(r(p_l)<`alpha');
	sum `tTest';
	return scalar tTest = r(mean);
	ranksum `var', by(`cat');
	g `ranksum' = 0;
	replace `ranksum' = 1 if(normprob(-abs(`r(z)'))<`alpha');
	sum `ranksum';
	return scalar ranksum = r(mean);
	permute `cat' ate=(r(mu_2)-r(mu_1)), reps(200) nodots nowarn nodrop left: 
	ttest `var', by(`cat');
	mat p = r(p);
	mat list p;
	g `permute' = 0;
	replace `permute' = 1 if(p[1,1]<`alpha');
	sum `permute';
	return scalar permute = r(mean);
end;

set obs 9;
g numObs = .;
g errorRateTTest = .;
g errorRateRanksum = .;
g errorRatePermute = .;
local j = 1;
foreach n in 2 4 8 12 15 20 50 100 500  {;
	preserve;
		simulate tTest=r(tTest) ranksum=r(ranksum) permute=r(permute), seed(010101) reps(1000): 
		testSmallTypeIError, n1(`n') n2(`n') alpha(.05);
		sum tTest;
		local tTest = r(mean);
		sum ranksum;
		local ranksum = r(mean);
		sum permute;
		local permute = r(mean);
	restore;
	replace errorRateTTest = `tTest' in `j';
	replace errorRateRanksum = `ranksum' in `j';
	replace errorRatePermute = `permute' in `j';
	replace numObs = `n' in `j';
	local j = `j' + 1;
};
	
gr tw
	(connected errorRateTTest numObs, mc(black) lc(black) sort)
	(connected errorRateRanksum numObs, mc(blue) lc(blue) sort)
	(connected errorRatePermute numObs, mc(red) lc(red) sort),
`graphr' leg(order(1 "t-test" 2 "Wilcoxon" 3 "Randomization") rows(1)) 
ysc(range(0 .1)) ylab(0(.025).1) xti("number of observations") yti("rejection rate") 
yline(.05, lc(gray) lw(thick)); 
gr export performenceTest_tTestVsRanksumVsRandomizationInference.pdf, replace;

* Assessment of test's H_0 rejection rate also possible with simplot command;
program drop _all;
program define simTTest, rclass;      
	drop _all;
    set obs 30;
    g x = rchi2(2);
    ttest x=2 in 1/10; 
	return scalar p10 = r(p);
    ttest x=2; 
    return scalar p30 = r(p);
end;

simulate p10=r(p10) p30=r(p30), reps(1000) seed(010101): simTTest; 

lab var p10 "N=10";
lab var p30 "N=30";
       
simpplot p10 p30, `graphr' main1opt(mcolor(red)) main2opt(mcolor(blue)); 
gr export simpplot_oneSampleTTest.pdf, replace;

*=================================================================================================;
* Statistical power evaluation 
*=================================================================================================;
* Check out Cameron/Trivedi: Microeconomics using Stata, pp.135ff, 140ff, 408ff;
program drop _all;
program powerCalculation, rclass;
	syntax [, numSim(integer 100) obs(integer 1) h0(real 0) ha(real 0) alpha(real .05)];
	tempname sim;
	postfile `sim' pvalues using powerResults, replace;
	forvalues i = 1/`numSim' {;
		drop _all;
		qui {;
			set obs `obs';
			g double var = rnormal();
			g y = `ha'*var + rchi2(1);
			reg y var;
			test var= `h0';
			scalar p = r(p);
			post `sim' (p);
		};
	};
	postclose `sim'; 
	use powerResults, clear;
	qui count if(pvalues < `alpha');
	return scalar power = r(N)/`numSim';
end;

set seed 10101;
set obs 200;
g power = .;
g numObs = .;
g HA = .;
local j = 1;
foreach n in 10 20 50 100 200 500 1000 {;
	forvalue HA = 2(.1)4 {;
		preserve;
			powerCalculation, numSim(1000) obs(`n') h0(2.00) ha(`HA') alpha(.05); 
		restore;
		replace power = r(power) in `j';
		replace numObs = `n' in `j';
		replace HA = `HA' in `j';
		local j = `j' + 1;
	};
};

use temp, clear;
gr tw 
	(connected power numObs if(HA==2), mc(red) lc(red))
	(connected power numObs if(HA==float(2.1)), mc(gs2) lc(gs2))
	(connected power numObs if(HA==float(2.2)), mc(gs4) lc(gs4))
	(connected power numObs if(HA==float(2.3)), mc(gs6) lc(gs6))
	(connected power numObs if(HA==float(2.4)), mc(gs8) lc(gs8))
	(connected power numObs if(HA==float(2.5)), mc(gs10) lc(gs10))
	(connected power numObs if(HA==float(3)), mc(gs12) lc(gs12))
	(connected power numObs if(HA==float(4)), mc(gs14) lc(gs14)),
`graphr' 
leg(order(1 "2" 2 "2.1" 3 "2.2" 4 "2.3" 5 "2.4" 6 "2.5" 7 "3" 8 "4") rows(1) subti(H_a))
xti(Number of observations) yti(Power);
gr export connected_powerVsNumObsVsHa.pdf, replace;

*=================================================================================================;
* Exit
*=================================================================================================;
exit;
*=================================================================================================;
*=================================================================================================;


